package com.sun.jmx.snmp.agent;

import javax.management.Notification;
import com.sun.jmx.snmp.SnmpValue;
import com.sun.jmx.snmp.SnmpInt;
import javax.management.MBeanNotificationInfo;
import javax.management.ListenerNotFoundException;
import java.util.Date;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import java.util.Enumeration;
import com.sun.jmx.snmp.SnmpVarBind;
import com.sun.jmx.snmp.SnmpStatusException;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import java.util.Hashtable;
import javax.management.ObjectName;
import java.util.Vector;
import com.sun.jmx.snmp.SnmpOid;
import java.io.Serializable;
import javax.management.NotificationBroadcaster;

public abstract class SnmpMibTable extends SnmpMibNode implements NotificationBroadcaster, Serializable
{
    protected int nodeId;
    protected SnmpMib theMib;
    protected boolean creationEnabled;
    protected SnmpTableEntryFactory factory;
    private int size;
    private static final int Delta = 16;
    private int tablecount;
    private int tablesize;
    private SnmpOid[] tableoids;
    private final Vector<Object> entries;
    private final Vector<ObjectName> entrynames;
    private Hashtable<NotificationListener, Vector<Object>> handbackTable;
    private Hashtable<NotificationListener, Vector<NotificationFilter>> filterTable;
    transient long sequenceNumber;
    
    public SnmpMibTable(SnmpMib theMib) {
        this.nodeId = 1;
        this.creationEnabled = false;
        this.factory = null;
        this.size = 0;
        this.tablecount = 0;
        this.tablesize = 16;
        this.tableoids = new SnmpOid[this.tablesize];
        this.entries = new Vector();
        this.entrynames = new Vector();
        this.handbackTable = new Hashtable();
        this.filterTable = new Hashtable();
        this.sequenceNumber = 0L;
        this.theMib = theMib;
        this.setCreationEnabled(false);
    }
    
    public abstract void createNewEntry(SnmpMibSubRequest p0, SnmpOid p1, int p2) throws SnmpStatusException;
    
    public abstract boolean isRegistrationRequired();
    
    public boolean isCreationEnabled() {
        return this.creationEnabled;
    }
    
    public void setCreationEnabled(boolean creationEnabled) {
        this.creationEnabled = creationEnabled;
    }
    
    public boolean hasRowStatus() {
        return false;
    }
    
    @Override
    public void get(SnmpMibSubRequest snmpMibSubRequest, int n) throws SnmpStatusException {
        int newEntry = snmpMibSubRequest.isNewEntry();
        if (!(newEntry == 0)) {
            Enumeration<SnmpVarBind> elements = snmpMibSubRequest.getElements();
            while (elements.hasMoreElements()) {
                snmpMibSubRequest.registerGetException((SnmpVarBind)elements.nextElement(), new SnmpStatusException(224));
            }
        }
        this.get(snmpMibSubRequest, snmpMibSubRequest.getEntryOid(), n + 1);
    }
    
    @Override
    public void check(SnmpMibSubRequest snmpMibSubRequest, int n) throws SnmpStatusException {
        SnmpOid entryOid = snmpMibSubRequest.getEntryOid();
        int rowAction = this.getRowAction(snmpMibSubRequest, entryOid, n + 1);
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "check", "Calling beginRowAction");
        }
        this.beginRowAction(snmpMibSubRequest, entryOid, n + 1, rowAction);
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "check", "Calling check for " + snmpMibSubRequest.getSize() + " varbinds");
        }
        this.check(snmpMibSubRequest, entryOid, n + 1);
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "check", "check finished");
        }
    }
    
    @Override
    public void set(SnmpMibSubRequest snmpMibSubRequest, int n) throws SnmpStatusException {
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "set", "Entering set");
        }
        SnmpOid entryOid = snmpMibSubRequest.getEntryOid();
        int rowAction = this.getRowAction(snmpMibSubRequest, entryOid, n + 1);
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "set", "Calling set for " + snmpMibSubRequest.getSize() + " varbinds");
        }
        this.set(snmpMibSubRequest, entryOid, n + 1);
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "set", "Calling endRowAction");
        }
        this.endRowAction(snmpMibSubRequest, entryOid, n + 1, rowAction);
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "set", "RowAction finished");
        }
    }
    
    public void addEntry(SnmpOid snmpOid, Object o) throws SnmpStatusException {
        this.addEntry(snmpOid, null, o);
    }
    
    public synchronized void addEntry(SnmpOid snmpOid, ObjectName obj, Object obj2) throws SnmpStatusException {
        if (this.isRegistrationRequired() && obj == null) {
            throw new SnmpStatusException(3);
        }
        if (this.size == 0) {
            this.insertOid(0, snmpOid);
            if (this.entries != null) {
                this.entries.addElement(obj2);
            }
            if (this.entrynames != null) {
                this.entrynames.addElement(obj);
            }
            ++this.size;
            if (this.factory != null) {
                try {
                    this.factory.addEntryCb(0, snmpOid, obj, obj2, this);
                }
                catch (SnmpStatusException ex3) {
                    this.removeOid(0);
                    if (this.entries != null) {
                        this.entries.removeElementAt(0);
                    }
                    if (this.entrynames != null) {
                        this.entrynames.removeElementAt(0);
                    }
                    throw ex3;
                }
            }
            this.sendNotification("jmx.snmp.table.entry.added", new Date().getTime(), obj2, obj);
            return;
        }
        else {
            int insertionPoint = this.getInsertionPoint(snmpOid, true);
            if (insertionPoint == this.size) {
                this.insertOid(this.tablecount, snmpOid);
                if (this.entries != null) {
                    this.entries.addElement(obj2);
                }
                if (this.entrynames != null) {
                    this.entrynames.addElement(obj);
                }
                ++this.size;
            }
            else {
                try {
                    this.insertOid(insertionPoint, snmpOid);
                    if (this.entries != null) {
                        this.entries.insertElementAt(obj2, insertionPoint);
                    }
                    if (this.entrynames != null) {
                        this.entrynames.insertElementAt(obj, insertionPoint);
                    }
                    ++this.size;
                }
                catch (ArrayIndexOutOfBoundsException ex) {}
            }
            if (this.factory != null) {
                try {
                    this.factory.addEntryCb(insertionPoint, snmpOid, obj, obj2, this);
                }
                catch (SnmpStatusException ex2) {
                    this.removeOid(insertionPoint);
                    if (this.entries != null) {
                        this.entries.removeElementAt(insertionPoint);
                    }
                    if (this.entrynames != null) {
                        this.entrynames.removeElementAt(insertionPoint);
                    }
                    throw ex2;
                }
            }
            this.sendNotification("jmx.snmp.table.entry.added", new Date().getTime(), obj2, obj);
            return;
        }
    }
    
    public synchronized void removeEntry(SnmpOid snmpOid, Object o) throws SnmpStatusException {
        int object = this.findObject(snmpOid);
        if (object == -1) {
            return;
        }
        this.removeEntry(object, o);
        return;
    }
    
    public void removeEntry(SnmpOid snmpOid) throws SnmpStatusException {
        int object = this.findObject(snmpOid);
        if (object == -1) {
            return;
        }
        this.removeEntry(object, null);
        return;
    }
    
    public synchronized void removeEntry(int n, Object element) throws SnmpStatusException {
        if (n == -1) {
            return;
        }
        if (n >= this.size) {
            return;
        }
        if (this.entries != null && this.entries.size() > n) {
            element = this.entries.elementAt(n);
            this.entries.removeElementAt(n);
        }
        ObjectName objectName = null;
        if (this.entrynames != null && this.entrynames.size() > n) {
            objectName = (ObjectName)this.entrynames.elementAt(n);
            this.entrynames.removeElementAt(n);
        }
        SnmpOid snmpOid = this.tableoids[n];
        this.removeOid(n);
        --this.size;
        if (element == null) {
        }
        if (this.factory != null) {
            this.factory.removeEntryCb(n, snmpOid, objectName, element, this);
        }
        this.sendNotification("jmx.snmp.table.entry.removed", new Date().getTime(), element, objectName);
        return;
    }
    
    public synchronized Object getEntry(SnmpOid snmpOid) throws SnmpStatusException {
        int object = this.findObject(snmpOid);
        if (object == -1) {
            throw new SnmpStatusException(224);
        }
        return this.entries.elementAt(object);
    }
    
    public synchronized ObjectName getEntryName(SnmpOid snmpOid) throws SnmpStatusException {
        int object = this.findObject(snmpOid);
        if (this.entrynames == null) {
            return null;
        }
        if (object == -1 || object >= this.entrynames.size()) {
            throw new SnmpStatusException(224);
        }
        return (ObjectName)this.entrynames.elementAt(object);
    }
    
    public Object[] getBasicEntries() {
        Object[] anArray = new Object[this.size];
        this.entries.copyInto(anArray);
        return anArray;
    }
    
    public int getSize() {
        return this.size;
    }
    
    @Override
    public synchronized void addNotificationListener(NotificationListener notificationListener, NotificationFilter obj, Object obj2) {
        if (notificationListener == null) {
            throw new IllegalArgumentException("Listener can't be null");
        }
        Vector<Object> value = (Vector)this.handbackTable.get(notificationListener);
        Vector<NotificationFilter> value2 = (Vector)this.filterTable.get(notificationListener);
        if (value == null) {
            value = new Vector();
            value2 = new Vector();
            this.handbackTable.put(notificationListener, value);
            this.filterTable.put(notificationListener, value2);
        }
        value.addElement(obj2);
        value2.addElement(obj);
        return;
    }
    
    @Override
    public synchronized void removeNotificationListener(NotificationListener key) throws ListenerNotFoundException {
        if ((Vector)this.handbackTable.get(key) == null) {
            throw new ListenerNotFoundException("listener");
        }
        this.handbackTable.remove(key);
        this.filterTable.remove(key);
        return;
    }
    
    @Override
    public MBeanNotificationInfo[] getNotificationInfo() {
        return new MBeanNotificationInfo[] { new MBeanNotificationInfo(new String[] { "jmx.snmp.table.entry.added", "jmx.snmp.table.entry.removed" }, "com.sun.jmx.snmp.agent.SnmpTableEntryNotification", "Notifications sent by the SnmpMibTable") };
    }
    
    public void registerEntryFactory(SnmpTableEntryFactory factory) {
        this.factory = factory;
    }
    
    protected boolean isRowStatus(SnmpOid snmpOid, long n, Object o) {
        return false;
    }
    
    protected int getRowAction(SnmpMibSubRequest snmpMibSubRequest, SnmpOid snmpOid, int n) throws SnmpStatusException {
        int newEntry = snmpMibSubRequest.isNewEntry();
        SnmpVarBind rowStatusVarBind = snmpMibSubRequest.getRowStatusVarBind();
        if (rowStatusVarBind == null) {
            if (newEntry && !this.hasRowStatus()) {
                return 4;
            }
            return 0;
        }
        else {
            try {
                return this.mapRowStatus(snmpOid, rowStatusVarBind, snmpMibSubRequest.getUserData());
            }
            catch (SnmpStatusException ex) {
                checkRowStatusFail(snmpMibSubRequest, ex.getStatus());
                return 0;
            }
        }
    }
    
    protected int mapRowStatus(SnmpOid snmpOid, SnmpVarBind snmpVarBind, Object o) throws SnmpStatusException {
        SnmpValue value = snmpVarBind.value;
        if ((value instanceof SnmpInt)) {
            return ((SnmpInt)value).intValue();
        }
        throw new SnmpStatusException(12);
    }
    
    protected SnmpValue setRowStatus(SnmpOid snmpOid, int n, Object o) throws SnmpStatusException {
        return null;
    }
    
    protected boolean isRowReady(SnmpOid snmpOid, Object o) throws SnmpStatusException {
        return true;
    }
    
    protected void checkRowStatusChange(SnmpMibSubRequest snmpMibSubRequest, SnmpOid snmpOid, int n, int n2) throws SnmpStatusException {
    }
    
    protected void checkRemoveTableRow(SnmpMibSubRequest snmpMibSubRequest, SnmpOid snmpOid, int n) throws SnmpStatusException {
    }
    
    protected void removeTableRow(SnmpMibSubRequest snmpMibSubRequest, SnmpOid snmpOid, int n) throws SnmpStatusException {
        this.removeEntry(snmpOid);
    }
    
    protected synchronized void beginRowAction(SnmpMibSubRequest snmpMibSubRequest, SnmpOid snmpOid, int n, int i) throws SnmpStatusException {
        int newEntry = snmpMibSubRequest.isNewEntry();
        switch (i) {
            case 0: {
                if (!(newEntry == 0)) {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "beginRowAction", "Failed to create row[" + snmpOid + "] : RowStatus = unspecified");
                    }
                    checkRowStatusFail(snmpMibSubRequest, 6);
                    break;
                }
                else {
                    break;
                }
            }
            case 4:
            case 5: {
                if (!(newEntry == 0)) {
                    if (this.isCreationEnabled()) {
                        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "beginRowAction", "Creating row[" + snmpOid + "] : RowStatus = createAndGo | createAndWait");
                        }
                        this.createNewEntry(snmpMibSubRequest, snmpOid, n);
                        break;
                    }
                    else {
                        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "beginRowAction", "Can't create row[" + snmpOid + "] : RowStatus = createAndGo | createAndWait but creation is disabled");
                        }
                        checkRowStatusFail(snmpMibSubRequest, 6);
                        break;
                    }
                }
                else {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "beginRowAction", "Can't create row[" + snmpOid + "] : RowStatus = createAndGo | createAndWait but row already exists");
                    }
                    checkRowStatusFail(snmpMibSubRequest, 12);
                    break;
                }
            }
            case 6: {
                if (!(newEntry == 0)) {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "beginRowAction", "Warning: can't destroy row[" + snmpOid + "] : RowStatus = destroy but row does not exist");
                    }
                }
                else if (!this.isCreationEnabled()) {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "beginRowAction", "Can't destroy row[" + snmpOid + "] : RowStatus = destroy but creation is disabled");
                    }
                    checkRowStatusFail(snmpMibSubRequest, 6);
                }
                this.checkRemoveTableRow(snmpMibSubRequest, snmpOid, n);
                break;
            }
            case 1:
            case 2: {
                if (!(newEntry == 0)) {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "beginRowAction", "Can't switch state of row[" + snmpOid + "] : specified RowStatus = active | notInService but row does not exist");
                    }
                    checkRowStatusFail(snmpMibSubRequest, 12);
                }
                this.checkRowStatusChange(snmpMibSubRequest, snmpOid, n, i);
                break;
            }
            default: {
                if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "beginRowAction", "Invalid RowStatus value for row[" + snmpOid + "] : specified RowStatus = " + i);
                }
                checkRowStatusFail(snmpMibSubRequest, 12);
                break;
            }
        }
    }
    
    protected void endRowAction(SnmpMibSubRequest snmpMibSubRequest, SnmpOid snmpOid, int n, int i) throws SnmpStatusException {
        int newEntry = snmpMibSubRequest.isNewEntry();
        Object userData = snmpMibSubRequest.getUserData();
        SnmpValue value = null;
        switch (i) {
            case 0: {
                break;
            }
            case 4: {
                if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "endRowAction", "Setting RowStatus to 'active' for row[" + snmpOid + "] : requested RowStatus = createAndGo");
                }
                value = this.setRowStatus(snmpOid, 1, userData);
                break;
            }
            case 5: {
                if (this.isRowReady(snmpOid, userData)) {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "endRowAction", "Setting RowStatus to 'notInService' for row[" + snmpOid + "] : requested RowStatus = createAndWait");
                    }
                    value = this.setRowStatus(snmpOid, 2, userData);
                    break;
                }
                else {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "endRowAction", "Setting RowStatus to 'notReady' for row[" + snmpOid + "] : requested RowStatus = createAndWait");
                    }
                    value = this.setRowStatus(snmpOid, 3, userData);
                    break;
                }
            }
            case 6: {
                if (!(newEntry == 0)) {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "endRowAction", "Warning: requested RowStatus = destroy, but row[" + snmpOid + "] does not exist");
                    }
                }
                else if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "endRowAction", "Destroying row[" + snmpOid + "] : requested RowStatus = destroy");
                }
                this.removeTableRow(snmpMibSubRequest, snmpOid, n);
                break;
            }
            case 1: {
                if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "endRowAction", "Setting RowStatus to 'active' for row[" + snmpOid + "] : requested RowStatus = active");
                }
                value = this.setRowStatus(snmpOid, 1, userData);
                break;
            }
            case 2: {
                if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "endRowAction", "Setting RowStatus to 'notInService' for row[" + snmpOid + "] : requested RowStatus = notInService");
                }
                value = this.setRowStatus(snmpOid, 2, userData);
                break;
            }
            default: {
                if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMibTable.class.getName(), "endRowAction", "Invalid RowStatus value for row[" + snmpOid + "] : specified RowStatus = " + i);
                }
                setRowStatusFail(snmpMibSubRequest, 12);
                break;
            }
        }
        if (value != null) {
            SnmpVarBind rowStatusVarBind = snmpMibSubRequest.getRowStatusVarBind();
            if (rowStatusVarBind != null) {
                rowStatusVarBind.value = value;
            }
        }
    }
    
    protected long getNextVarEntryId(SnmpOid snmpOid, long nextVarEntryId, Object o, int n) throws SnmpStatusException {
        do {
            nextVarEntryId = this.getNextVarEntryId(snmpOid, nextVarEntryId, o);
        } while (this.skipEntryVariable(snmpOid, nextVarEntryId, o, n));
        return nextVarEntryId;
    }
    
    protected boolean skipEntryVariable(SnmpOid snmpOid, long n, Object o, int n2) {
        return false;
    }
    
    protected SnmpOid getNextOid(SnmpOid snmpOid, Object o) throws SnmpStatusException {
        if (this.size == 0) {
            throw new SnmpStatusException(224);
        }
        if (this.tableoids[this.tablecount - 1].equals(snmpOid)) {
            throw new SnmpStatusException(224);
        }
        int insertionPoint = this.getInsertionPoint(snmpOid, false);
        if (insertionPoint > -1 && insertionPoint < this.size) {
            SnmpOid snmpOid2;
            try {
                snmpOid2 = this.tableoids[insertionPoint];
            }
            catch (ArrayIndexOutOfBoundsException ex) {
                throw new SnmpStatusException(224);
            }
            return snmpOid2;
        }
        throw new SnmpStatusException(224);
    }
    
    protected SnmpOid getNextOid(Object o) throws SnmpStatusException {
        if (this.size == 0) {
            throw new SnmpStatusException(224);
        }
        return this.tableoids[0];
    }
    
    protected abstract long getNextVarEntryId(SnmpOid p0, long p1, Object p2) throws SnmpStatusException;
    
    protected abstract void validateVarEntryId(SnmpOid p0, long p1, Object p2) throws SnmpStatusException;
    
    protected abstract boolean isReadableEntryId(SnmpOid p0, long p1, Object p2) throws SnmpStatusException;
    
    protected abstract void get(SnmpMibSubRequest p0, SnmpOid p1, int p2) throws SnmpStatusException;
    
    protected abstract void check(SnmpMibSubRequest p0, SnmpOid p1, int p2) throws SnmpStatusException;
    
    protected abstract void set(SnmpMibSubRequest p0, SnmpOid p1, int p2) throws SnmpStatusException;
    
    SnmpOid getNextOid(long[] array, int n, Object o) throws SnmpStatusException {
        return this.getNextOid(new SnmpEntryOid(array, n), o);
    }
    
    static void checkRowStatusFail(SnmpMibSubRequest snmpMibSubRequest, int n) throws SnmpStatusException {
        snmpMibSubRequest.registerCheckException(snmpMibSubRequest.getRowStatusVarBind(), new SnmpStatusException(n));
    }
    
    static void setRowStatusFail(SnmpMibSubRequest snmpMibSubRequest, int n) throws SnmpStatusException {
        snmpMibSubRequest.registerSetException(snmpMibSubRequest.getRowStatusVarBind(), new SnmpStatusException(n));
    }
    
    @Override
    final synchronized void findHandlingNode(SnmpVarBind snmpVarBind, long[] array, int n, SnmpRequestTree snmpRequestTree) throws SnmpStatusException {
        int length = array.length;
        if (snmpRequestTree == null) {
            throw new SnmpStatusException(5);
        }
        if (n >= length) {
            throw new SnmpStatusException(6);
        }
        if (array[n] != (long)this.nodeId) {
            throw new SnmpStatusException(6);
        }
        if (n + 2 >= length) {
            throw new SnmpStatusException(6);
        }
        SnmpEntryOid snmpEntryOid = new SnmpEntryOid(array, n + 2);
        Object userData = snmpRequestTree.getUserData();
        int contains = this.contains(snmpEntryOid, userData);
        if (contains == 0) {
            if (!snmpRequestTree.isCreationAllowed()) {
                throw new SnmpStatusException(224);
            }
            if (!this.isCreationEnabled()) {
                throw new SnmpStatusException(6);
            }
        }
        long n2 = array[n + 1];
        if (!(contains == 0)) {
            this.validateVarEntryId(snmpEntryOid, n2, userData);
        }
        if (snmpRequestTree.isSetRequest() && this.isRowStatus(snmpEntryOid, n2, userData)) {
            snmpRequestTree.add(this, n, snmpEntryOid, snmpVarBind, (boolean)(contains == 0), snmpVarBind);
        }
        else {
            snmpRequestTree.add(this, n, snmpEntryOid, snmpVarBind, (boolean)(contains == 0));
        }
        return;
    }
    
    @Override
    final synchronized long[] findNextHandlingNode(SnmpVarBind snmpVarBind, long[] array, int n, int n2, SnmpRequestTree snmpRequestTree, AcmChecker acmChecker) throws SnmpStatusException {
        int length = array.length;
        if (snmpRequestTree == null) {
            throw new SnmpStatusException(225);
        }
        Object userData = snmpRequestTree.getUserData();
        int requestPduVersion = snmpRequestTree.getRequestPduVersion();
        long n3 = -1L;
        if (n >= length) {
            array = new long[] { (long)this.nodeId };
            n = 0;
            length = 1;
        }
        else {
            if (array[n] > (long)this.nodeId) {
                throw new SnmpStatusException(225);
            }
            if (array[n] < (long)this.nodeId) {
                array = new long[] { (long)this.nodeId };
                n = 0;
                length = 0;
            }
            else if (n + 1 < length) {
                n3 = array[n + 1];
            }
        }
        SnmpOid snmpOid;
        if (n == length - 1) {
            snmpOid = this.getNextOid(userData);
            n3 = this.getNextVarEntryId(snmpOid, n3, userData, requestPduVersion);
        }
        else if (n == length - 2) {
            snmpOid = this.getNextOid(userData);
            if (this.skipEntryVariable(snmpOid, n3, userData, requestPduVersion)) {
                n3 = this.getNextVarEntryId(snmpOid, n3, userData, requestPduVersion);
            }
        }
        else {
            try {
                snmpOid = this.getNextOid(array, n + 2, userData);
                if (this.skipEntryVariable(snmpOid, n3, userData, requestPduVersion)) {
                    throw new SnmpStatusException(225);
                }
            }
            catch (SnmpStatusException ex) {
                snmpOid = this.getNextOid(userData);
                n3 = this.getNextVarEntryId(snmpOid, n3, userData, requestPduVersion);
            }
        }
        return this.findNextAccessibleOid(snmpOid, snmpVarBind, array, n2, snmpRequestTree, acmChecker, userData, n3);
    }
    
    private long[] findNextAccessibleOid(SnmpOid snmpOid, SnmpVarBind snmpVarBind, long[] array, int n, SnmpRequestTree snmpRequestTree, AcmChecker acmChecker, Object o, long nextVarEntryId) throws SnmpStatusException {
        int requestPduVersion = snmpRequestTree.getRequestPduVersion();
        while (snmpOid != null && nextVarEntryId != -1L) {
            try {
                if (!this.isReadableEntryId(snmpOid, nextVarEntryId, o)) {
                    throw new SnmpStatusException(225);
                }
                long[] longValue = snmpOid.longValue(false);
                int length = longValue.length;
                long[] array2 = new long[n + 2 + length];
                array2[0] = -1L;
                System.arraycopy(longValue, 0, array2, n + 2, length);
                array2[n] = (long)this.nodeId;
                array2[n + 1] = nextVarEntryId;
                acmChecker.add(n, array2, n, length + 2);
                try {
                    acmChecker.checkCurrentOid();
                    snmpRequestTree.add(this, n, snmpOid, snmpVarBind, false);
                    return array2;
                }
                catch (SnmpStatusException ex) {
                    snmpOid = this.getNextOid(snmpOid, o);
                }
                finally {
                    acmChecker.remove(n, length + 2);
                }
            }
            catch (SnmpStatusException ex2) {
                snmpOid = this.getNextOid(o);
                nextVarEntryId = this.getNextVarEntryId(snmpOid, nextVarEntryId, o, requestPduVersion);
            }
            if (snmpOid != null && nextVarEntryId != -1L) {
                continue;
            }
            throw new SnmpStatusException(225);
        }
        throw new SnmpStatusException(225);
    }
    
    final void validateOid(long[] array, int n) throws SnmpStatusException {
        if (n + 2 >= array.length) {
            throw new SnmpStatusException(224);
        }
        if (array[n] != (long)this.nodeId) {
            throw new SnmpStatusException(225);
        }
        return;
    }
    
    private synchronized void sendNotification(Notification notification) {
        Enumeration<NotificationListener> keys = this.handbackTable.keys();
        while (keys.hasMoreElements()) {
            NotificationListener notificationListener = (NotificationListener)keys.nextElement();
            Vector<Object> vector = (Vector)this.handbackTable.get(notificationListener);
            Enumeration<Object> elements = ((Vector)this.filterTable.get(notificationListener)).elements();
            Enumeration<Object> elements2 = vector.elements();
            while (elements2.hasMoreElements()) {
                Object nextElement = elements2.nextElement();
                NotificationFilter notificationFilter = (NotificationFilter)elements.nextElement();
                if (notificationFilter == null || notificationFilter.isNotificationEnabled(notification)) {
                    notificationListener.handleNotification(notification, nextElement);
                }
                else {
                    continue;
                }
            }
        }
    }
    
    private void sendNotification(String s, long n, Object o, ObjectName objectName) {
        synchronized (this) {
            ++this.sequenceNumber;
        }
        this.sendNotification(new SnmpTableEntryNotification(s, this, this.sequenceNumber, n, o, objectName));
    }
    
    protected boolean contains(SnmpOid snmpOid, Object o) {
        return this.findObject(snmpOid) > -1;
    }
    
    private int findObject(SnmpOid snmpOid) {
        int i = 0;
        int n = this.size - 1;
        int n2 = i + (n - i) / 2;
        while (i <= n) {
            SnmpOid snmpOid2 = this.tableoids[n2];
            int compareTo = snmpOid.compareTo(snmpOid2);
            if (compareTo == 0) {
                return n2;
            }
            if (snmpOid.equals(snmpOid2)) {
                return n2;
            }
            if (compareTo > 0) {
                i = n2 + 1;
            }
            else {
                n = n2 - 1;
            }
            n2 = i + (n - i) / 2;
        }
        return -1;
    }
    
    private int getInsertionPoint(SnmpOid snmpOid, boolean b) throws SnmpStatusException {
        int i = 0;
        int n = this.size - 1;
        int n2 = i + (n - i) / 2;
        while (i <= n) {
            int compareTo = snmpOid.compareTo(this.tableoids[n2]);
            if (compareTo == 0) {
                if (b) {
                    throw new SnmpStatusException(17, n2);
                }
                return n2 + 1;
            }
            else {
                if (compareTo > 0) {
                    i = n2 + 1;
                }
                else {
                    n = n2 - 1;
                }
                n2 = i + (n - i) / 2;
            }
        }
        return n2;
    }
    
    private void removeOid(int n) {
        if (n >= this.tablecount) {
            return;
        }
        if (n < 0) {
            return;
        }
        int tablecount = this.tablecount - 1;
        this.tablecount = tablecount;
        int n2 = tablecount - n;
        this.tableoids[n] = null;
        if (n2 > 0) {
            System.arraycopy(this.tableoids, n + 1, this.tableoids, n, n2);
        }
        this.tableoids[this.tablecount] = null;
        return;
    }
    
    private void insertOid(int tablecount, SnmpOid snmpOid) {
        if (tablecount >= this.tablesize || this.tablecount == this.tablesize) {
            SnmpOid[] tableoids = this.tableoids;
            this.tablesize += 16;
            this.tableoids = new SnmpOid[this.tablesize];
            if (tablecount > this.tablecount) {
                tablecount = this.tablecount;
            }
            if (tablecount < 0) {
                tablecount = 0;
            }
            int n = this.tablecount - tablecount;
            if (tablecount > 0) {
                System.arraycopy(tableoids, 0, this.tableoids, 0, tablecount);
            }
            if (n > 0) {
                System.arraycopy(tableoids, tablecount, this.tableoids, tablecount + 1, n);
            }
        }
        else if (tablecount < this.tablecount) {
            System.arraycopy(this.tableoids, tablecount, this.tableoids, tablecount + 1, this.tablecount - tablecount);
        }
        this.tableoids[tablecount] = snmpOid;
        ++this.tablecount;
    }
}
